==> ifrit_3.0.0.orig.tar.gz: ifrit-3.0.0/core/idatareader.h <==
/*=========================================================================

  Program:   Ionization FRont Interactive Tool (IFRIT)
  Language:  C++


Copyright (c) 2002-2006 Nick Gnedin 
All rights reserved.

This file may be distributed and/or modified under the terms of the
GNU General Public License version 2 as published by the Free Software
Foundation and appearing in the file LICENSE.GPL included in the
packaging of this file.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

=========================================================================*/


#ifndef IDATAREADER_H
#define IDATAREADER_H


#include "iobject.h"
#include "iviewmodulecomponent.h"
#include "ierrorstatusholder.h"
#include "ideletable.h"


#include "iarray.h"


class iBasicParticlesDataSubject;
class iDataInfo;
class iDataLimits;
class iDataReaderHelper;
class iDataSubject;
class iDataType;
class iDirectory;
class iFile;
class iProgressEventObserver;
class iUniformScalarsDataSubject;
class iUniformVectorsDataSubject;
class iUniformTensorsDataSubject;

class vtkDataSet;
class vtkPolyData;
class vtkStructuredPoints;
 

class iDataReader : public iObject, public iErrorStatusHolder, public iViewModuleComponent, public iDeletable
{

	friend class iDataReaderHelper;
	friend class iExtensionFactory;

public:

	struct VisitedFile
	{
		iString Name;
		const iDataSubject *Subject;
	};

	static iDataReader* New(iViewModule *vm);
	static const iObjectType& Type();
	
	//
	//  Non-static members
	//
	IOBJECT_DECLARE_GETSET(ScaledDimension,mScaledDim,int);
	//virtual void SetScaledDimension(int v);
	//inline int GetScaledDimension() const { return mScaledDim; }

	IOBJECT_DECLARE_GETSET1(CellToPointMode,int);
	//virtual void SetCellToPointMode(int m);
	//inline int GetCellToPointMode() const { return mCellToPointMode; }

	IOBJECT_DECLARE_GETSET2(ParticlesOrderInFileIsAttribute,bool);
	//virtual void SetParticlesOrderInFileIsAttribute(bool s);
	//bool GetParticlesOrderInFileIsAttribute() const;

	IOBJECT_DECLARE_GETSET2(ScalarFieldCalculatorFunction,const iString&);
	//virtual void SetScalarFieldCalculatorFunction(const iString &s);
	//const iString& GetScalarFieldCalculatorFunction() const;

	IOBJECT_DECLARE_GETSET2(ScalarFieldCalculatorOutputVariable,int);
	//virtual void SetScalarFieldCalculatorOutputVariable(int n);
	//int GetScalarFieldCalculatorOutputVariable() const;

	IOBJECT_DECLARE_GETSET2(ParticlesDownsampleMode,int);
	//virtual void SetParticlesDownsampleMode(int v);
	//int GetParticlesDownsampleMode() const;

	IOBJECT_DECLARE_GETSET2(ParticlesDownsampleFactor,int);
	//virtual void SetParticlesDownsampleFactor(int v);
	//int GetParticlesDownsampleFactor() const;

	IOBJECT_DECLARE_GETSET1(VoxelLocation,int);
	//virtual void SetVoxelLocation(int s);
	//inline bool GetVoxelLocation() const { return mVoxelLocation; }

	IOBJECT_DECLARE_GETSET1(BoundaryConditions,int);
	//virtual void SetBoundaryConditions(int s);
	//inline int GetBoundaryConditions() const { return mBoundaryConditions; }

	virtual void SetDataShift(int d, double f); 
	inline double GetDataShift(int d) const { if(d>=0 && d<3) return mDataShift[d]; else return 0.0; }
	static const iObjectKey& KeyShiftData();

	//
	//  Hidden (undocumented) keys
	//
	static const iObjectKey& KeyRecordLength(); 

	//
	//  Special "action" keys
	//
	static const iObjectKey& KeyEraseData(); 
	static const iObjectKey& KeyLoadData(); 

	//
	//  Special "query" keys 
	//
	static const iObjectKey& KeyErrorMessage();

	inline bool IsSet() const { return mSetSubjects.Size() > 0; }
	static const iObjectKey& KeyIsSet();

	//
	//  Loading of data
	//
	virtual void LoadFile(const iDataType &type, const iString &fname, bool dryrun = false, bool savename = true);
    void LoadRecord(int rec, int skip = 0, bool dryrun = false, bool savename = false);

	//
	//  Operation on data
	//
	virtual bool IsThereData(const iDataType &type) const;
	virtual bool IsThereData(const iDataInfo &info) const;

	virtual void EraseData(const iDataType &type);
	virtual void DetachData(const iDataType &type);
	virtual void ReloadData(const iDataType &type);

	virtual void ResetPipeline();

	iDataSubject* GetSubject(const iDataType &type) const;
	iDataLimits* GetLimits(const iDataType &type) const;

	//
	//  Access to included DataType(s)
	//
	inline int GetNumberOfDataTypes() const { return mSubjects.Size(); }
	const iDataType& GetDataType(int i) const;

	//
	//  Generic GetOutput
	//
	vtkDataSet* GetOutput(const iDataType &type) const;

	inline bool IsFileAnimatable() const { return (mCurrentRecord >= 0); }
	inline int GetRecordNumber() const { return mCurrentRecord; }

	virtual void ShiftData(double *dx = 0);

	virtual float GetMemorySize() const;
	
	//
	//  Functions for file manipulation
	//
	const iString& GetLastFileSetName() const;
	bool IsSetMember(const iString &fname) const;
	const iString GetFileSetName(int rec) const;
	const iDataType& GetFileSetDataType(int member = 0) const;

	const iString& GetLastFileName(const iDataType &type) const;
	inline const iString& GetLastFileName() const { return mLastFileName; }
	inline const iString& GetLastAttemptedFileName() const { return mLastAttemptedFileName; }

	inline const VisitedFile* GetVisitedFilesList() const { return &(mVisitedFilesList[0]); }
	inline int GetNumberOfVisitedFiles() const { return mVisitedFilesList.Size(); }

	virtual iProgressEventObserver* GetProgressEventObserver(const iDataType &type);

	virtual void PackCompleteState(iString &s) const;
	virtual void UnPackCompleteState(const iString &s);

	void LoadFileByStringKey(const iDataType &type, int envkey, iString ws[2]);

protected:
	
	iDataReader(iViewModule *vm);
	virtual ~iDataReader();

	virtual void PackStateBody(iString &s) const;
	virtual void UnPackStateBody(const iString &s);

	virtual void InsertIntoVisitedFilesList(const iDataSubject *subject, const iString &file);

	//
	//  Inheritable loading engine: inherit those to allow extension data types to
	//  join in a set
	//
	virtual int DissectFileName(const iString &fname, iString &root, iString &record, iString &suffix) const;
	virtual void BreakSet();
	virtual bool IsAborted(const iDataType &type) const;

	void InsertStringIntoVisitedFilesList(const iDataSubject *subject, const iString &string);
	iString GetRecordAsString(int rec) const;

	bool mTwoCopies;
	int mRecordLength, mCellToPointMode;

	double mDataShift[3];
	iString mMode;
	int mScaledDim, mVoxelLocation, mBoundaryConditions;

	iDataSubject *mPeriodicityLeader;
	iSearchableArray<iDataSubject*> mSetSubjects;
	iArray<iDataSubject*> mSubjects;
	iUniformScalarsDataSubject *mUniformScalarsSubject;
	iUniformVectorsDataSubject *mUniformVectorsSubject;
	iUniformTensorsDataSubject *mUniformTensorsSubject;
	iBasicParticlesDataSubject *mBasicParticlesSubject;

	int mCurrentRecord, mLastRecord;

	iString mLastFileName, mLastAttemptedFileName;
	iArray<VisitedFile> mVisitedFilesList;
	iDirectory *mDirectory;
};

#endif

==> ifrit_3.0.5.orig.tar.gz: ifrit-3.0.5/core/idatareader.h <==
/*=========================================================================

  Program:   Ionization FRont Interactive Tool (IFRIT)
  Language:  C++


Copyright (c) 2002-2006 Nick Gnedin 
All rights reserved.

This file may be distributed and/or modified under the terms of the
GNU General Public License version 2 as published by the Free Software
Foundation and appearing in the file LICENSE.GPL included in the
packaging of this file.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

=========================================================================*/


#ifndef IDATAREADER_H
#define IDATAREADER_H


#include "iextendableobject.h"
#include "iviewmodulecomponent.h"
#include "ierrorstatusholder.h"
#include "ideletable.h"


#include "iarray.h"


class iBasicParticlesDataSubject;
class iDataInfo;
class iDataLimits;
class iDataReaderHelper;
class iDataSubject;
class iDataType;
class iDirectory;
class iFile;
class iProgressEventObserver;
class iUniformScalarsDataSubject;
class iUniformVectorsDataSubject;
class iUniformTensorsDataSubject;

class vtkDataSet;
class vtkPolyData;
class vtkStructuredPoints;


class iDataReader : public iExtendableObject, public iErrorStatusHolder, public iViewModuleComponent, public iDeletable
{

	friend class iDataReaderHelper;
	friend class iExtensionFactory;

public:

	IOBJECT_DECLARE_SELF(iDataReader,iExtendableObject);

	struct VisitedFile
	{
		iString Name;
		const iDataSubject *Subject;
	};

	static iDataReader* New(iViewModule *vm);
	static const iObjectType& Type();
	
	//
	//  Non-static members
	//
	IOBJECT_DECLARE_GETSET(ScaledDimension,mScaledDim,int);
	//virtual void SetScaledDimension(int v);
	//inline int GetScaledDimension() const { return mScaledDim; }

	IOBJECT_DECLARE_GETSET1(CellToPointMode,int);
	//virtual void SetCellToPointMode(int m);
	//inline int GetCellToPointMode() const { return mCellToPointMode; }

	IOBJECT_DECLARE_GETSET2(ParticlesOrderInFileIsAttribute,bool);
	//virtual void SetParticlesOrderInFileIsAttribute(bool s);
	//bool GetParticlesOrderInFileIsAttribute() const;

	IOBJECT_DECLARE_GETSET2(ScalarFieldCalculatorFunction,const iString&);
	//virtual void SetScalarFieldCalculatorFunction(const iString &s);
	//const iString& GetScalarFieldCalculatorFunction() const;

	IOBJECT_DECLARE_GETSET2(ScalarFieldCalculatorOutputVariable,int);
	//virtual void SetScalarFieldCalculatorOutputVariable(int n);
	//int GetScalarFieldCalculatorOutputVariable() const;

	IOBJECT_DECLARE_GETSET2(ParticlesDownsampleMode,int);
	//virtual void SetParticlesDownsampleMode(int v);
	//int GetParticlesDownsampleMode() const;

	IOBJECT_DECLARE_GETSET2(ParticlesDownsampleFactor,int);
	//virtual void SetParticlesDownsampleFactor(int v);
	//int GetParticlesDownsampleFactor() const;

	IOBJECT_DECLARE_GETSET1(VoxelLocation,int);
	//virtual void SetVoxelLocation(int s);
	//inline bool GetVoxelLocation() const { return mVoxelLocation; }

	IOBJECT_DECLARE_GETSET1(BoundaryConditions,int);
	//virtual void SetBoundaryConditions(int s);
	//inline int GetBoundaryConditions() const { return mBoundaryConditions; }

	void SetDataShift(int d, double f); 
	inline double GetDataShift(int d) const { if(d>=0 && d<3) return mDataShift[d]; else return 0.0; }
	static const iObjectKey& KeyShiftData();

	//
	//  Special "action" keys
	//
	static const iObjectKey& KeyEraseData(); 
	static const iObjectKey& KeyLoadData(); 

	//
	//  Special "query" keys 
	//
	static const iObjectKey& KeyErrorMessage();

	inline bool IsSet() const { return mSetSubjects.Size() > 0; }
	static const iObjectKey& KeyIsSet();

	//
	//  Loading of data
	//
	void LoadFile(const iDataType &type, const iString &fname, bool dryrun = false, bool savename = true);
    void LoadRecord(int rec, int skip = 0, bool dryrun = false, bool savename = false);

	//
	//  Operation on data
	//
	bool IsThereData(const iDataType &type) const;
	bool IsThereData(const iDataInfo &info) const;

	void EraseData(const iDataType &type);
	void DetachData(const iDataType &type);
	void ReloadData(const iDataType &type);

	void ResetPipeline();

	iDataSubject* GetSubject(const iDataType &type) const;
	iDataLimits* GetLimits(const iDataType &type) const;

	//
	//  Access to included DataType(s)
	//
	inline int GetNumberOfDataTypes() const { return mSubjects.Size(); }
	const iDataType& GetDataType(int i) const;

	//
	//  Generic GetOutput
	//
	vtkDataSet* GetOutput(const iDataType &type) const;

	inline bool IsFileAnimatable() const { return (mCurrentRecord >= 0); }
	inline int GetRecordNumber() const { return mCurrentRecord; }

	virtual void ShiftData(double *dx = 0);

	virtual float GetMemorySize() const;
	
	//
	//  Functions for file manipulation
	//
	const iString& GetLastFileSetName() const;
	bool IsAnotherSetLeader(const iString &fname) const;
	bool IsCurrentSetMember(const iString &fname) const;
	const iString GetFileSetName(int rec) const;
	const iDataType& GetFileSetDataType(int member = 0) const;

	const iString& GetLastFileName(const iDataType &type) const;
	inline const iString& GetLastFileName() const { return mLastFileName; }
	inline const iString& GetLastAttemptedFileName() const { return mLastAttemptedFileName; }

	inline const VisitedFile* GetVisitedFilesList() const { return &(mVisitedFilesList[0]); }
	inline int GetNumberOfVisitedFiles() const { return mVisitedFilesList.Size(); }

	virtual iProgressEventObserver* GetProgressEventObserver(const iDataType &type);

	virtual void PackCompleteState(iString &s) const;
	virtual void UnPackCompleteState(const iString &s);

	void LoadFileByStringKey(const iDataType &type, int envkey, iString ws[2]);

protected:
	
	iDataReader(iViewModule *vm);
	virtual ~iDataReader();

	virtual void ExtendableObjectPackStateBody(iString &s) const;
	virtual void ExtendableObjectUnPackStateBody(const iString &s);

	void InsertIntoVisitedFilesList(const iDataSubject *subject, const iString &file);
	void BreakSet();
	bool IsAborted(const iDataType &type) const;
	bool IsSetLeader(const iDataSubject *subject) const;

	void InsertStringIntoVisitedFilesList(const iDataSubject *subject, const iString &string);

	bool mTwoCopies, mInExtensionUpdate;
	int mCellToPointMode;

	double mDataShift[3];
	iString mMode;
	int mScaledDim, mVoxelLocation, mBoundaryConditions;

	iDataSubject *mPeriodicityLeader;
	iSearchableArray<iDataSubject*> mSetSubjects;
	iArray<iDataSubject*> mSubjects;
	iUniformScalarsDataSubject *mUniformScalarsSubject;
	iUniformVectorsDataSubject *mUniformVectorsSubject;
	iUniformTensorsDataSubject *mUniformTensorsSubject;
	iBasicParticlesDataSubject *mBasicParticlesSubject;

	int mCurrentRecord;

	iString mLastFileName, mLastAttemptedFileName;
	iArray<VisitedFile> mVisitedFilesList;
	iDirectory *mDirectory;
};


class iDataReaderExtension : public iObjectExtension
{

	IOBJECTEXTENSION_DECLARE_ABSTRACT(iDataReader);

public:

	iDataSubject* AddNewSubject();

	virtual void AfterLoadFile(const iDataType &type, const iString &fname, bool dryrun = false) = 0;
	virtual bool IsUsingData(const iDataType &type) const = 0;

protected:

	virtual iDataSubject* CreateSubject(int n) const = 0;

	int mNextSubjectToCreate;
};

#endif
